﻿using Core;

namespace Projekt_Calcan_Conze_Export.Models;

internal class Billing
{
    public Billing(
        DateOnly startDate,
        DateOnly endDate,
        string customerNumber,
        string customerName,
        List<BillingPosition> positions)
    {
        this.StartDate = startDate;
        this.EndDate = endDate;
        this.CustomerNumber = customerNumber;
        this.CustomerName = customerName;
        this.Positions = positions;
    }

    public DateOnly StartDate { get; }

    public DateOnly EndDate { get; }

    public string CustomerNumber { get; }

    public string CustomerName { get; }

    public double TotalAmount => this.Positions.Sum(p => p.TotalAmount);

    public List<BillingPosition> Positions { get; }

    public IEnumerable<string> ToCsvLines()
    {
        List<string> lines =
        [
            this.StartDate.ToString(Constants.DateFormat)
            + Constants.Separator
            + this.EndDate.ToString(Constants.DateFormat)
            + Constants.Separator
            + this.CustomerNumber
            + Constants.Separator
            + this.CustomerName
            + Constants.Separator
            + this.TotalAmount.ToString(Constants.MoneyFormat)

        ];

        lines.AddRange(
            this.Positions
                .Select(
                    position =>
                        position.Description
                        + Constants.Separator
                        + position.BaseAmount.ToString(Constants.MoneyFormat)
                        + Constants.Separator
                        + position.Count
                        + Constants.Separator
                        + position.TotalAmount.ToString(Constants.MoneyFormat)
                        + Constants.Separator));

        return lines;
    }
}

internal class BillingPosition
{
    public BillingPosition(string description, double baseAmount, int count)
    {
        this.Description = description;
        this.BaseAmount = baseAmount;
        this.Count = count;
    }

    public string Description { get; }

    public double BaseAmount { get; }

    public int Count { get; set; }

    public double TotalAmount => this.BaseAmount * this.Count;
}